from django.conf import settings
from django.test import TestCase
from django.utils.translation import activate, get_language
from django.utils.translation import gettext as _
from translation.tests.models import TranslatableTestModel as TestModel


class TranslatableModelGetFieldTranslationTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_raises_exception_if_field_is_not_in_translatable_property(self):
        with self.assertRaises(KeyError) as context:
            self.model = TestModel.objects.create(title={"en": "Welcome :)", "ar": "اهلا"})
            self.model.get_field_translation('created_at', settings.LANGUAGE_ARABIC)
            self.assertEquals(context.exception, _('"created_at" is not translatable field'))

    def test_it_raises_exception_if_field_value_is_string(self):
        with self.assertRaises(ValueError) as context:
            self.model = TestModel.objects.create(title="Welcome :)")
            self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
            self.assertEquals(context.exception, _('"title" value is string, expected to be json'))

    def test_it_returns_the_required_locale(self):
        self.model = TestModel.objects.create(title={"en": "Welcome :)", "ar": "اهلا"})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "اهلا")

    def test_it_returns_the_value_of_fallback_locale_if_requested_locale_does_not_exists(self):
        self.model = TestModel.objects.create(title={"en": "Welcome :)"})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "Welcome :)")

    def test_it_returns_the_value_of_any_available_locale_if_requested_locale_and_fallback_locale_does_not_exists(self):
        self.model = TestModel.objects.create(title={"es": "Welcome :)"})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "Welcome :)")

    def test_it_returns_empty_string_if_field_value_has_no_values(self):
        self.model = TestModel.objects.create(title={})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "")

    def test_it_returns_empty_string_if_field_value_has_None_value(self):
        self.model = TestModel.objects.create()
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "")


class TranslatableModelGetTranslatedObjectTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_returns_all_translatable_fields_translated_into_required_locale(self):
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        model.get_translated_object(settings.LANGUAGE_ARABIC)
        self.assertEquals(model.title, "اهلا")
        self.assertEquals(model.title, "اهلا")

    def test_it_returns_all_translatable_fields_translated_into_current_locale_if_locale_is_not_provided(self):
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        model.get_translated_object()
        self.assertEquals(model.title, "Welcome :)")
        self.assertEquals(model.title, "Welcome :)")


class TranslatableModelSetTranslationTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_hard_sets_field_translation_for_target_field(self):
        model = TestModel.objects.create(title={"en": "Welcome :)"})
        model.set_translation('title', settings.LANGUAGE_ARABIC, "اهلا")

        # hard check
        model.refresh_from_db()
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ARABIC), "اهلا")
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")

    def test_it_soft_sets_field_translation_for_target_field(self):
        model = TestModel.objects.create(title={"en": "Welcome :)"})
        model.set_translation('title', settings.LANGUAGE_ARABIC, "اهلا", soft=True)

        # soft check
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ARABIC), "اهلا")
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")

        # hard check
        model.refresh_from_db()
        self.assertTrue(settings.LANGUAGE_ARABIC not in model.title)
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")

    def test_it_sets_the_field_value_to_the_language_code_if_field_value_is_string(self):
        model = TestModel.objects.create(title="Welcome :)")
        model.set_translation('title', settings.LANGUAGE_ARABIC, "اهلا")
        # hard check
        model.refresh_from_db()
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ARABIC), "اهلا")

    def test_it_throws_exception_if_field_is_not_translatable(self):
        with self.assertRaises(KeyError) as context:
            model = TestModel.objects.create(title="Welcome :)")
            model.set_translation('created_at', settings.LANGUAGE_ARABIC, "اهلا")
            self.assertEquals(context.exception, _('"created_at" is not translatable field'))

    def test_it_throws_exception_if_locale_is_not_in_settings_LANGUAGES_KEYS(self):
        with self.assertRaises(KeyError) as context:
            model = TestModel.objects.create(title="Welcome :)")
            model.set_translation('title', 'ccc', "اهلا")
            self.assertEquals(context.exception, _('"ccc" is not translatable field'))


class TranslatableModelTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_contains_translated_property_for_each_translatable_field(self):
        model = TestModel.objects.create(title={"en": "Welcome :)"})
        self.assertTrue(
            all([hasattr(model, 'translated_{}'.format(field)) for field, __ in model.translatable.items()]))

    def test_it_passes_the_field_value_as_it_is_if_the_value_is_string(self):
        model = TestModel.objects.create(title="Welcome :)")
        self.assertEquals(model.translated_title, "Welcome :)")

    def test_it_returns_translated_fields_with_current_locale_if_exists_in_field(self):
        activate(settings.LANGUAGE_ARABIC)
        model = TestModel.objects.create(title={"en": "Welcome :)", "ar": "اهلا"},
                                         description={"en": "Welcome :)", "ar": "اهلا"})
        self.assertEquals(model.translated_title, "اهلا")

    def test_it_returns_translated_field_with_the_value_of_fallback_locale_if_requested_locale_does_not_exists(self):
        activate(settings.LANGUAGE_ARABIC)
        model = TestModel.objects.create(title={"en": "Welcome :)"}, description={"en": "Welcome :)"})
        self.assertEquals(model.translated_title, "Welcome :)")

    def test_it_returns_translated_field_with_the_value_of_any_available_locale_if_requested_locale_and_fallback_locale_does_not_exists(
            self):
        model = TestModel.objects.create(title={"es": "Welcome :)"}, description={"es": "Welcome :)"})
        self.assertEquals(model.translated_title, "Welcome :)")

    def test_it_returns_empty_string_if_field_value_has_no_values(self):
        model = TestModel.objects.create()
        self.assertEquals(model.translated_title, "")

    def test_each_translated_field_has_the_correct_value(self):
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        self.assertTrue(
            all([model.get_field_translation(field, get_language()) == getattr(model, "translated_{}".format(field))
                 for field, __ in model.translatable.items()]))

    def test_each_translated_field_has_the_correct_value_in_different_language(self):
        activate(settings.LANGUAGE_ARABIC)
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        self.assertTrue(
            all([model.get_field_translation(field, get_language()) == getattr(model, "translated_{}".format(field))
                 for field, __ in model.translatable.items()]))



from django import template
from django.conf import settings

register = template.Library()


@register.simple_tag(name="get_field_original_translation")
def get_field_original_translation(_o, field):
    return _o.get_field_translation(field, settings.MAIN_LANGUAGE)


register.filter('get_field_original_translation', get_field_original_translation)



from django.conf import settings
from django.utils.translation import gettext as _, get_language
from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class TranslatableModelSerializer(serializers.ModelSerializer):
    def to_internal_value(self, data):
        for field_name in self.Meta.model.translatable:
            if field_name in data:
                if isinstance(data[field_name], dict):
                    for locale, value in data[field_name].items():
                        if locale not in settings.LANGUAGES_KEYS:
                            raise ValidationError({field_name: _("'{}' is not in supported locales".format(locale))})
                else:
                    if 'locale' in self.context and self.context.get('locale', None):
                        data[field_name] = {self.context['locale']: data[field_name]}
                    else:
                        data[field_name] = {get_language(): data[field_name]}
        return super().to_internal_value(data)

    def to_representation(self, instance):
        representation = super().to_representation(instance)
        for field, __ in instance.translatable.items():
            if field in representation:
                if isinstance(representation[field], dict):
                    if get_language() in representation[field]:
                        representation[field] = representation[field][get_language()]
                    else:
                        if settings.FALLBACK_LOCALE in representation[field]:
                            representation[field] = representation[field][settings.FALLBACK_LOCALE]
                        else:
                            representation[field] = ''
        return representation

    def update(self, instance, validated_data):
        for field_name, __ in self.Meta.model.translatable.items():
            if field_name in validated_data:
                for locale, value in validated_data[field_name].items():
                    instance.set_translation(field_name, locale, value, soft=True)
                validated_data.pop(field_name)
        return super(TranslatableModelSerializer, self).update(instance, validated_data)

    class Meta:
        model = None
        fields = '__all__'



from django.conf import settings
from django.test import TestCase
from django.utils.translation import activate, get_language
from translation.serializers import TranslatableModelSerializer
from translation.tests.models import TranslatableTestModel


class TestSerializer(TranslatableModelSerializer):
    class Meta:
        model = TranslatableTestModel
        fields = '__all__'


class TranslatableModelSerializerTestCase(TestCase):
    def setUp(self):
        self.data = {'title': 'title', 'description': 'description'}
        self.serializer = TestSerializer(data=self.data)
        self.serializer.is_valid()
        self.serializer.save()
        self.test_obj = TranslatableTestModel.objects.get(pk=1)

    def test_it_turns_all_translatable_fields_into_json(self):
        self.assertEquals(self.test_obj.translated_title, 'title')

    def test_it_keeps_un_translatable_fields_as_it_is(self):
        self.assertNotIsInstance(self.test_obj.created_at, dict)

    def test_it_set_field_to_current(self):
        ((locale, value),) = self.test_obj.title.items()
        self.assertEquals(locale, get_language())

    def test_it_set_fields_as_it_is_when_insert_json(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        serializer.is_valid()
        serializer.save()
        self.assertEquals(serializer.instance.title, data['title'])

    def test_it_return_validation_error_when_insert_unsupported_locale(self):
        data = {'title': {'en': 'title', 'su': 'asdca'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertEquals(set(serializer.errors.keys()), {'title'})

    def test_it_return_field_by_current_locale(self):
        self.assertEquals(self.serializer.data['title'], 'title')

    def test_it_returns_fall_back_locale_when_there_is_no_value_for_current_locale(self):
        activate(next(locale for locale in settings.LANGUAGES_KEYS if locale != settings.LANGUAGE_CODE))
        self.assertEquals(self.serializer.data['title'], 'title')

    def test_it_return_current_locale_when_activate_other_locale(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        serializer.is_valid()
        serializer.save()
        activate(next(locale for locale in settings.LANGUAGES_KEYS if locale != get_language()))
        self.assertEquals(serializer.data['title'], data['title'][get_language()])
        activate(next(locale for locale in settings.LANGUAGES_KEYS if locale != get_language()))

    def test_it_return_field_by_current_locale_when_insert_json(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        serializer.is_valid()
        serializer.save()
        self.assertEquals(serializer.data['title'], data['title'][settings.LANGUAGE_CODE])
        self.assertEquals(serializer.data['description'], data['description'][settings.LANGUAGE_CODE])

    def test_it_updates_instance(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj)
        serializer.is_valid()
        serializer.save()
        self.test_obj.refresh_from_db()
        self.assertEquals(self.test_obj.title, data['title'])

    def test_it_updates_specified_locale_value(self):
        data = {'title': 'title', 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj, context={'locale': 'en'})
        serializer.is_valid()
        serializer.save()
        self.assertEquals(self.test_obj.title['en'], data['title']['en'])

    def test_it_append_supported_locale_value_if_does_not_exist(self):
        other_locale = next(locale for locale in settings.LANGUAGES_KEYS if locale != settings.LANGUAGE_CODE)
        data = {'title': {other_locale: 'other locale title'}, 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj)
        serializer.is_valid()
        serializer.save()
        title = {**data['title'], settings.LANGUAGE_CODE: self.data['title'][settings.LANGUAGE_CODE]}
        self.assertEquals(self.test_obj.title, title)

    def test_it_update_locale_values_when_it_exists(self):
        data = {'title': {settings.LANGUAGE_CODE: 'other title'}, 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj)
        serializer.is_valid()
        serializer.save()
        self.assertEquals(data['title'][settings.LANGUAGE_CODE], self.test_obj.title[settings.LANGUAGE_CODE])



from django.conf import settings
from django.db import models
from django.utils.translation import gettext as _, get_language


class TranslatableModel(models.Model):
    class Meta:
        abstract = True

    translatable = {}

    def get_field_translation(self, field, locale):
        if field not in self.translatable:
            raise KeyError(_('"{}" is not translatable field'.format(field)))

        field_value = getattr(self, field)

        if isinstance(field_value, str):
            raise ValueError(_('"{}" value is string, expected to be json'.format(field)))

        if not field_value:
            return ''

        if locale in field_value:
            return field_value[locale]
        elif settings.FALLBACK_LOCALE in field_value:
            return field_value[settings.FALLBACK_LOCALE]
        elif bool(field_value):
            available_locale = list(field_value.keys())[0]
            return field_value[available_locale]
        else:
            return ""

    def get_translated_object(self, locale=None):
        if not locale:
            locale = get_language()

        for field in self.translatable:
            field_value = self.get_field_translation(field, locale)
            setattr(self, field, field_value)

    def set_translation(self, field, locale, value, soft=False):
        if field not in self.translatable:
            raise KeyError(_('"{}" is not translatable_field'.format(field)))

        if locale not in settings.LANGUAGES_KEYS:
            raise KeyError(_('"{}" is not translatable_field'.format(locale)))

        field_value = getattr(self, field)
        if isinstance(field_value, str):
            field_value = {settings.LANGUAGE_CODE: field_value}

        field_value.update({locale: value})
        setattr(self, field, field_value)
        if not soft:
            self.save()

    def __init__(self, *args, **kwargs):
        super(TranslatableModel, self).__init__(*args, **kwargs)

        for field, __ in self.translatable.items():
            field_value = getattr(self, field)

            current_locale = get_language()
            if field_value:
                if isinstance(field_value, str):
                    translated_value = field_value
                elif current_locale in field_value:
                    translated_value = field_value[current_locale]
                elif settings.FALLBACK_LOCALE in field_value:
                    translated_value = field_value[settings.FALLBACK_LOCALE]
                elif bool(field_value):
                    available_locale = list(field_value.keys())[0]
                    translated_value = field_value[available_locale]
                else:
                    translated_value = ""
            else:
                translated_value = ""

            setattr(self, 'translated_{}'.format(field), translated_value)



from django import forms
from django.conf import settings

from django.forms.widgets import TextInput, Textarea
from django.test import TestCase, Client
from django.urls import reverse, resolve
from django.utils.translation import activate, get_language
from django.utils.translation import gettext as _
from translation.forms import TranslationForm, TranslatableModelForm
from translation.serializers import TranslatableModelSerializer
from translation.tests.models import TranslatableTestModel as TestModel, TranslatableTestModel
from rest_framework.test import APIClient
from rest_framework import status


class TranslatableModelGetFieldTranslationTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_raises_exception_if_field_is_not_in_translatable_property(self):
        with self.assertRaises(KeyError) as context:
            self.model = TestModel.objects.create(title={"en": "Welcome :)", "ar": "اهلا"})
            self.model.get_field_translation('created_at', settings.LANGUAGE_ARABIC)
            self.assertEquals(context.exception, _('"created_at" is not translatable field'))

    def test_it_raises_exception_if_field_value_is_string(self):
        with self.assertRaises(ValueError) as context:
            self.model = TestModel.objects.create(title="Welcome :)")
            self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
            self.assertEquals(context.exception, _('"title" value is string, expected to be json'))

    def test_it_returns_the_required_locale(self):
        self.model = TestModel.objects.create(title={"en": "Welcome :)", "ar": "اهلا"})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "اهلا")

    def test_it_returns_the_value_of_fallback_locale_if_requested_locale_does_not_exists(self):
        self.model = TestModel.objects.create(title={"en": "Welcome :)"})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "Welcome :)")

    def test_it_returns_the_value_of_any_available_locale_if_requested_locale_and_fallback_locale_does_not_exists(self):
        self.model = TestModel.objects.create(title={"es": "Welcome :)"})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "Welcome :)")

    def test_it_returns_empty_string_if_field_value_has_no_values(self):
        self.model = TestModel.objects.create(title={})
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "")

    def test_it_returns_empty_string_if_field_value_has_None_value(self):
        self.model = TestModel.objects.create()
        translated_title = self.model.get_field_translation('title', settings.LANGUAGE_ARABIC)
        self.assertEquals(translated_title, "")


class TranslatableModelGetTranslatedObjectTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_returns_all_translatable_fields_translated_into_required_locale(self):
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        model.get_translated_object(settings.LANGUAGE_ARABIC)
        self.assertEquals(model.title, "اهلا")
        self.assertEquals(model.title, "اهلا")

    def test_it_returns_all_translatable_fields_translated_into_current_locale_if_locale_is_not_provided(self):
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        model.get_translated_object()
        self.assertEquals(model.title, "Welcome :)")
        self.assertEquals(model.title, "Welcome :)")


class TranslatableModelSetTranslationTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_hard_sets_field_translation_for_target_field(self):
        model = TestModel.objects.create(title={"en": "Welcome :)"})
        model.set_translation('title', settings.LANGUAGE_ARABIC, "اهلا")

        # hard check
        model.refresh_from_db()
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ARABIC), "اهلا")
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")

    def test_it_soft_sets_field_translation_for_target_field(self):
        model = TestModel.objects.create(title={"en": "Welcome :)"})
        model.set_translation('title', settings.LANGUAGE_ARABIC, "اهلا", soft=True)

        # soft check
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ARABIC), "اهلا")
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")

        # hard check
        model.refresh_from_db()
        self.assertTrue(settings.LANGUAGE_ARABIC not in model.title)
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")

    def test_it_sets_the_field_value_to_the_language_code_if_field_value_is_string(self):
        model = TestModel.objects.create(title="Welcome :)")
        model.set_translation('title', settings.LANGUAGE_ARABIC, "اهلا")
        # hard check
        model.refresh_from_db()
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ENGLISH), "Welcome :)")
        self.assertEquals(model.get_field_translation('title', settings.LANGUAGE_ARABIC), "اهلا")

    def test_it_throws_exception_if_field_is_not_translatable(self):
        with self.assertRaises(KeyError) as context:
            model = TestModel.objects.create(title="Welcome :)")
            model.set_translation('created_at', settings.LANGUAGE_ARABIC, "اهلا")
            self.assertEquals(context.exception, _('"created_at" is not translatable field'))

    def test_it_throws_exception_if_locale_is_not_in_settings_LANGUAGES_KEYS(self):
        with self.assertRaises(KeyError) as context:
            model = TestModel.objects.create(title="Welcome :)")
            model.set_translation('title', 'ccc', "اهلا")
            self.assertEquals(context.exception, _('"ccc" is not translatable field'))


class TranslatableModelTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_contains_translated_property_for_each_translatable_field(self):
        model = TestModel.objects.create(title={"en": "Welcome :)"})
        self.assertTrue(
            all([hasattr(model, 'translated_{}'.format(field)) for field, __ in model.translatable.items()]))

    def test_it_passes_the_field_value_as_it_is_if_the_value_is_string(self):
        model = TestModel.objects.create(title="Welcome :)")
        self.assertEquals(model.translated_title, "Welcome :)")

    def test_it_returns_translated_fields_with_current_locale_if_exists_in_field(self):
        activate(settings.LANGUAGE_ARABIC)
        model = TestModel.objects.create(title={"en": "Welcome :)", "ar": "اهلا"},
                                         description={"en": "Welcome :)", "ar": "اهلا"})
        self.assertEquals(model.translated_title, "اهلا")

    def test_it_returns_translated_field_with_the_value_of_fallback_locale_if_requested_locale_does_not_exists(self):
        activate(settings.LANGUAGE_ARABIC)
        model = TestModel.objects.create(title={"en": "Welcome :)"}, description={"en": "Welcome :)"})
        self.assertEquals(model.translated_title, "Welcome :)")

    def test_it_returns_translated_field_with_the_value_of_any_available_locale_if_requested_locale_and_fallback_locale_does_not_exists(
            self):
        model = TestModel.objects.create(title={"es": "Welcome :)"}, description={"es": "Welcome :)"})
        self.assertEquals(model.translated_title, "Welcome :)")

    def test_it_returns_empty_string_if_field_value_has_no_values(self):
        model = TestModel.objects.create()
        self.assertEquals(model.translated_title, "")

    def test_each_translated_field_has_the_correct_value(self):
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        self.assertTrue(
            all([model.get_field_translation(field, get_language()) == getattr(model, "translated_{}".format(field))
                 for field, __ in model.translatable.items()]))

    def test_each_translated_field_has_the_correct_value_in_different_language(self):
        activate(settings.LANGUAGE_ARABIC)
        model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        self.assertTrue(
            all([model.get_field_translation(field, get_language()) == getattr(model, "translated_{}".format(field))
                 for field, __ in model.translatable.items()]))


class TranslationFormTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

        self.instance = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_updates_the_same_instance_of_the_model(self):
        """
        the form must update the same instance and must not create any new records
        """
        form = TranslationForm(
            model_class=TestModel,
            instance=self.instance,
            locale="ar",
            data={"title": "اهلا", "description": "اهلا"}
        )
        instance = form.save()
        self.assertEquals(instance.pk, self.instance.pk)

    def test_throws_exception_if_model_class_is_not_provided(self):
        """
        model_class must be provided to form
        """
        with self.assertRaises(KeyError) as context:
            TranslationForm(locale="ar", instance=self.instance)
        self.assertTrue("Form missing 1 required key word argument: 'model_class'" in str(context.exception))

    def test_throws_exception_if_instance_is_not_provided(self):
        """
        instance must be provided to form
        """
        with self.assertRaises(KeyError) as context:
            TranslationForm(locale="ar", model_class=TestModel)
        self.assertTrue("Form missing 1 required key word argument: 'instance'" in str(context.exception))

    def test_throws_exception_if_locale_is_not_provided(self):
        """
        locale must be provided to form
        """
        with self.assertRaises(KeyError) as context:
            TranslationForm(model_class=TestModel, instance=self.instance)
        self.assertTrue("Form missing 1 required key word argument: 'locale'" in str(context.exception))

    def test_it_returns_initial_values_with_available_locale_values(self):
        form = TranslationForm(model_class=TestModel, instance=self.instance, locale="ar")
        self.assertEquals(form.initial['title'], "اهلا")
        self.assertEquals(form.initial['description'], "اهلا")

    def test_it_returns_fields_and_widgets_based_on_model_translatable_attributes(self):
        form = TranslationForm(model_class=TestModel, instance=self.instance, locale="ar")
        self.assertIsInstance(form.fields['title'], forms.CharField)
        self.assertIsInstance(form.fields['title'].widget, TextInput)
        self.assertIsInstance(form.fields['description'], forms.CharField)
        self.assertIsInstance(form.fields['description'].widget, Textarea)

    def test_it_updates_fields_without_overriding_other_locales(self):
        self.instance = TestModel.objects.create(title={"en": "Welcome :)"}, description={"en": "Welcome :)"})
        form = TranslationForm(
            model_class=TestModel,
            instance=self.instance,
            locale="ar",
            data={"title": "اهلا", "description": "اهلا"}
        )
        self.assertTrue(form.is_valid())
        form.save()
        self.instance.refresh_from_db()

        self.assertEquals(self.instance.title['en'], "Welcome :)")
        self.assertEquals(self.instance.title['ar'], "اهلا")
        self.assertEquals(self.instance.description['en'], "Welcome :)")
        self.assertEquals(self.instance.description['ar'], "اهلا")

    def test_it_throws_exception_if_input_is_invalid_locales(self):
        """
        throws exception if the locale is not a valid locale
        """
        with self.assertRaises(ValueError) as context:
            TranslationForm(
                model_class=TestModel,
                instance=self.instance,
                locale="not-a-language",
                data={"title": "اهلا", "description": "اهلا"}
            )
            self.assertTrue("input locale must be included in application languages" in str(context.exception))


class AdminTranslationViewTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

        self.client = Client()
        self.model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        self.url = reverse("translate:admin-view",
                           args=["tests", "TranslatableTestModel", self.model.pk]) + "?next=/test/"

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_uses_translate_template_in_GET_request(self):
        """
        method: GET
        template: translate.html
        """
        self.response = self.client.get(self.url)
        self.assertTemplateUsed(self.response, "translate.html")

    def test_context_has_list_of_languages_in_GET_request(self):
        """
        method: GET
        context must have list of languages
        """
        self.response = self.client.get(self.url)
        self.assertIn("languages", self.response.context)

    def test_languages_is_all_locales_except_the_app_locale_in_GET_request(self):
        """
        method: GET
        it returns the languages key with the value of all locales except the locale stored in
        settings.LANGUAGE_CODE
        """
        self.response = self.client.get(self.url)
        languages = dict(settings.LANGUAGES)
        languages.pop(settings.LANGUAGE_CODE)
        self.assertEquals(languages, self.response.context['languages'])

    def test_context_has_next_in_GET_request(self):
        """
          method: GET
          context must have next link
        """
        self.response = self.client.get(self.url)
        self.assertIn("next", self.response.context)

    def test_it_throws_exception_if_next_is_not_in_url_in_GET_request(self):
        """
        method: GET
        context must have next link
        """
        with self.assertRaises(KeyError) as context:
            self.response = self.client.get(reverse("translate:admin-view", args=["admin", "testmodel", 1, 'ar']))
            self.assertTrue("key next must be included in request" in str(context.exception))

    def test_it_uses_the_next_locale_if_locale_is_not_provided_in_url(self):
        url = reverse("translate:admin-view",
                      args=["tests", "TranslatableTestModel", self.model.pk]) + "?next=/test/"
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_context_has_target_locale_in_GET_request(self):
        """
        method: GET
        context must have list of languages
        """
        self.response = self.client.get(self.url)
        self.assertIn("locale", self.response.context)

    def test_context_has_app_label_in_GET_request(self):
        """
        method: GET
        context must have app_label
        """
        self.response = self.client.get(self.url)
        self.assertIn("app_label", self.response.context)

    def test_context_has_model_in_GET_request(self):
        """
        method: GET
        context must have model
        """
        self.response = self.client.get(self.url)
        self.assertIn("model", self.response.context)

    def test_context_has_object_in_GET_request(self):
        """
        method: GET
        context must have object
        """
        self.response = self.client.get(self.url)
        self.assertIn("object", self.response.context)

    def test_object_must_be_an_instance_of_model_in_GET_request(self):
        """
        method: GET
        context must have object
        """
        self.response = self.client.get(self.url)
        self.assertEquals(self.response.context['object'], self.model)

    def test_context_has_form_in_GET_request(self):
        """
        method: GET
        context must have form
        """
        self.response = self.client.get(self.url)
        self.assertIn("form", self.response.context)

    def test_form_is_an_instance_of_AdminTranslationForm_in_GET_request(self):
        """
        method: GET
        context must have form
        """
        self.response = self.client.get(self.url)
        self.assertIsInstance(self.response.context["form"], TranslationForm)

    def test_it_throws_exception_if_locale_is_not_in_request_in_POST_request(self):
        """
        method: GET
        context must have next link
        """
        with self.assertRaises(KeyError) as context:
            self.client.post(self.url, {"title": "اهلا وسهلا", "description": "اهلا وسهلا", "save": ""})

    def test_it_throws_exception_if_next_is_not_in_url_in_POST_request(self):
        """
        method: POST
        context must have next link
        """
        with self.assertRaises(KeyError) as context:
            self.client.post(self.url, {
                "title": "اهلا وسهلا",
                "description": "اهلا وسهلا",
                "save": ""})
            self.assertTrue("key next must be included in request" in str(context.exception))

    def test_it_updates_model_value_on_submit_in_POST_request(self):
        self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "description": "اهلا وسهلا",
            "locale": settings.LANGUAGE_ARABIC,
            "next": "/test/",
            "save": ""
        })
        self.model.refresh_from_db()
        self.assertEquals("اهلا وسهلا", self.model.title['ar'])

    def test_it_redirects_to_next_url_if_save_is_not_in_request_in_POST_request(self):
        self.response = self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "description": "اهلا وسهلا",
            "locale": settings.LANGUAGE_ARABIC,
            "next": "/",
        })
        self.assertRedirects(self.response, '/')
        self.assertEquals(self.response.status_code, 302)

    def test_it_redirects_to_same_url_if_save_in_POST_request(self):
        self.response = self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "description": "اهلا وسهلا",
            "next": "/test/",
            "locale": settings.LANGUAGE_ARABIC,
            "save": ""})
        self.assertRedirects(self.response, self.url)
        self.assertEquals(self.response.status_code, 302)

    def test_it_renders_the_same_requests_with_errors_if_data_is_invalid_in_POST_request(self):
        self.response = self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "next": "/test/",
            "locale": settings.LANGUAGE_ARABIC,
            "save": ""})
        self.assertTemplateUsed(self.response, "translate.html")
        self.assertEquals(self.response.status_code, 422)


class TestModelForm(TranslatableModelForm):
    class Meta:
        model = TestModel
        fields = "__all__"


class TranslatableModelFormTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_returns_fields_and_widgets_based_on_model_translatable_attributes(self):
        form = TestModelForm(data={"title": "Welcome :)", "description": "Welcome :)"})
        self.assertIsInstance(form.fields['title'], forms.CharField)
        self.assertIsInstance(form.fields['title'].widget, TextInput)
        self.assertIsInstance(form.fields['description'], forms.CharField)
        self.assertIsInstance(form.fields['description'].widget, Textarea)

    def test_it_returns_field_initial_value_in_original_locale_only(self):
        instance = TestModel.objects.create(
            title={"en": "Welcome from title:)", "ar": "اهلا فى العنوان"},
            description={"en": "Welcome from description:)", "ar": "اهلا فى الوصف"})
        form = TestModelForm(instance=instance)
        self.assertEquals(form.instance.title, "Welcome from title:)")

    def test_it_saves_translatable_value_with_settings_language_code_in_creation_of_new_record(self):
        """
        in creation of new record the original field value is None
        """
        form = TestModelForm(data={"title": "Welcome from title:)", "description": "Welcome from description:)"})
        self.assertTrue(form.is_valid())
        form.save()

        instance = TestModel.objects.get(pk=1)
        self.assertEquals(instance.title['en'], "Welcome from title:)")
        self.assertEquals(instance.description['en'], "Welcome from description:)")

    def test_it_updates_translatable_values_of_settings_default_locale(self):
        """
        in instance update it saves the new values
        """
        TestModel.objects.create(title={"en": "Welcome from title:)"}, description={"en": "Welcome from description:)"})

        form = TestModelForm(data={"title": "Welcome!!", "description": "Welcome Dude!!"},
                             instance=TestModel.objects.get(pk=1))
        self.assertTrue(form.is_valid())
        form.save()
        instance = TestModel.objects.get(pk=1)

        self.assertEquals(instance.title['en'], "Welcome!!")
        self.assertEquals(instance.description['en'], "Welcome Dude!!")

    def test_it_maintains_other_locales_on_instance_update(self):
        """
        in instance update it saves the new values  without changing other locales values
        """
        instance = TestModel.objects.create(
            title={"en": "Welcome from title:)", "ar": "اهلا فى العنوان"},
            description={"en": "Welcome from description:)", "ar": "اهلا فى الوصف"})

        form = TestModelForm(data={"title": "Welcome!!", "description": "Welcome Dude!!"}, instance=instance)

        self.assertTrue(form.is_valid())
        form.save()

        self.assertEquals(instance.title['ar'], "اهلا فى العنوان")
        self.assertEquals(instance.description['ar'], "اهلا فى الوصف")
        self.assertEquals(instance.title['en'], "Welcome!!")
        self.assertEquals(instance.description['en'], "Welcome Dude!!")


class TestSerializer(TranslatableModelSerializer):
    class Meta:
        model = TranslatableTestModel
        fields = '__all__'


class TranslatableModelSerializerTestCase(TestCase):
    def setUp(self):
        self.data = {'title': 'title', 'description': 'description'}
        self.serializer = TestSerializer(data=self.data)
        self.serializer.is_valid()
        self.serializer.save()
        self.test_obj = TranslatableTestModel.objects.get(pk=1)

    def test_it_turns_all_translatable_fields_into_json(self):
        self.assertEquals(self.test_obj.translated_title, 'title')

    def test_it_keeps_un_translatable_fields_as_it_is(self):
        self.assertNotIsInstance(self.test_obj.created_at, dict)

    def test_it_set_field_to_current(self):
        ((locale, value),) = self.test_obj.title.items()
        self.assertEquals(locale, get_language())

    def test_it_set_fields_as_it_is_when_insert_json(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        serializer.is_valid()
        serializer.save()
        self.assertEquals(serializer.instance.title, data['title'])

    def test_it_return_validation_error_when_insert_unsupported_locale(self):
        data = {'title': {'en': 'title', 'su': 'asdca'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertEquals(set(serializer.errors.keys()), {'title'})

    def test_it_return_field_by_current_locale(self):
        self.assertEquals(self.serializer.data['title'], 'title')

    def test_it_returns_fall_back_locale_when_there_is_no_value_for_current_locale(self):
        activate(next(locale for locale in settings.LANGUAGES_KEYS if locale != settings.LANGUAGE_CODE))
        self.assertEquals(self.serializer.data['title'], 'title')

    def test_it_return_current_locale_when_activate_other_locale(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        serializer.is_valid()
        serializer.save()
        activate(next(locale for locale in settings.LANGUAGES_KEYS if locale != get_language()))
        self.assertEquals(serializer.data['title'], data['title'][get_language()])
        activate(next(locale for locale in settings.LANGUAGES_KEYS if locale != get_language()))

    def test_it_return_field_by_current_locale_when_insert_json(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data)
        serializer.is_valid()
        serializer.save()
        self.assertEquals(serializer.data['title'], data['title'][settings.LANGUAGE_CODE])
        self.assertEquals(serializer.data['description'], data['description'][settings.LANGUAGE_CODE])

    def test_it_updates_instance(self):
        data = {'title': {'en': 'title', 'ar': 'عنوان'}, 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj)
        serializer.is_valid()
        serializer.save()
        self.test_obj.refresh_from_db()
        self.assertEquals(self.test_obj.title, data['title'])

    def test_it_updates_specified_locale_value(self):
        data = {'title': 'title', 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj, context={'locale': 'en'})
        serializer.is_valid()
        serializer.save()
        self.assertEquals(self.test_obj.title['en'], data['title']['en'])

    def test_it_append_supported_locale_value_if_does_not_exist(self):
        other_locale = next(locale for locale in settings.LANGUAGES_KEYS if locale != settings.LANGUAGE_CODE)
        data = {'title': {other_locale: 'other locale title'}, 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj)
        serializer.is_valid()
        serializer.save()
        title = {**data['title'], settings.LANGUAGE_CODE: self.data['title'][settings.LANGUAGE_CODE]}
        self.assertEquals(self.test_obj.title, title)

    def test_it_update_locale_values_when_it_exists(self):
        data = {'title': {settings.LANGUAGE_CODE: 'other title'}, 'description': 'description'}
        serializer = TestSerializer(data=data, instance=self.test_obj)
        serializer.is_valid()
        serializer.save()
        self.assertEquals(data['title'][settings.LANGUAGE_CODE], self.test_obj.title[settings.LANGUAGE_CODE])


class TranslationAPIViewTestCase(TestCase):
    def setUp(self):
        self.test_model = TestModel.objects.create(title={settings.LANGUAGE_CODE: "title"},
                                                   description={"en": "description"})
        self.client = APIClient()
        self.url = reverse('translate:api-view', kwargs={"app_label": 'tests',
                                                             'model': 'TranslatableTestModel',
                                                             'pk': '1'})

    def test_it_translate_current_locale_value_when_locale_is_none(self):
        data = {"title": {settings.LANGUAGE_CODE: "other title"}}
        self.client.post(self.url, data=data, format='json')
        self.test_model.refresh_from_db()
        self.assertEquals(self.test_model.title, data['title'])

    def test_it_returns_422_status_code_when_add_invalid_data(self):
        data = {"title": {'sd': "other title"}}
        response = self.client.post(self.url, data=data, format='json')
        self.assertEquals(response.status_code, status.HTTP_422_UNPROCESSABLE_ENTITY)

    def test_it_translate_chosen_locale(self):
        locale = next(locale for locale in settings.LANGUAGES_KEYS if settings.LANGUAGE_CODE != locale)
        data = {"title": {locale: "{} title".format(locale)}, 'locale': locale}
        response = self.client.post(self.url, data=data, format='json')
        self.test_model.refresh_from_db()
        self.assertEquals(self.test_model.title[locale], data["title"][locale])

    def test_it_updates_existed_locales_values(self):
        data = {"title": {"en": "en title", "ar": "ar title"}}
        response = self.client.post(self.url, data=data, format='json')
        self.test_model.refresh_from_db()
        self.assertEquals(self.test_model.title, data['title'])



# Generated by Django 3.2 on 2022-01-11 13:05

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyNewTest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=20, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='TranslatableTestModel',
            fields=[
                ('created_at', models.DateTimeField(auto_created=True, blank=True, default=django.utils.timezone.now)),
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('title', models.JSONField(blank=True, null=True)),
                ('description', models.JSONField(blank=True, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MyTest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=20, null=True)),
                ('new_test', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='tests.mynewtest')),
            ],
        ),
    ]



from django.urls import path, include

from .views import TranslationAPIView, AdminTranslateView

app_name = 'translation'

urlpatterns = [
    path('translate/api/<str:app_label>/<str:model>/<str:pk>/', TranslationAPIView.as_view(),
         name='translate'),
    path('translate/api/<str:app_label>/<str:model>/<str:pk>/<str:locale>/', TranslationAPIView.as_view(),
         name='translate_locale'),
    path('translate/<str:app_label>/<str:model>/<str:pk>/', AdminTranslateView.as_view(), name='translate_form'),
    path('translate/<str:app_label>/<str:model>/<str:pk>/<str:locale>/', AdminTranslateView.as_view(),
         name='translate_form_locale'),
]



from django.conf import settings
from django.test import TestCase, Client
from django.urls import reverse
from django.utils.translation import activate
from translation.forms import TranslationForm
from translation.tests.models import TranslatableTestModel as TestModel


class AdminTranslationViewTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

        self.client = Client()
        self.model = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )
        self.url = reverse("translation:translate_form",
                           args=["tests", "TranslatableTestModel", self.model.pk]) + "?next=/test/"

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_uses_translate_template_in_GET_request(self):
        """
        method: GET
        template: translate.html
        """
        self.response = self.client.get(self.url)
        self.assertTemplateUsed(self.response, "translation/translate.html")

    def test_context_has_list_of_languages_in_GET_request(self):
        """
        method: GET
        context must have list of languages
        """
        self.response = self.client.get(self.url)
        self.assertIn("languages", self.response.context)

    def test_languages_is_all_locales_except_the_app_locale_in_GET_request(self):
        """
        method: GET
        it returns the languages key with the value of all locales except the locale stored in
        settings.LANGUAGE_CODE
        """
        self.response = self.client.get(self.url)
        languages = dict(settings.LANGUAGES)
        languages.pop(settings.LANGUAGE_CODE)
        self.assertEquals(languages, self.response.context['languages'])

    def test_context_has_next_in_GET_request(self):
        """
          method: GET
          context must have next link
        """
        self.response = self.client.get(self.url)
        self.assertIn("next", self.response.context)

    def test_it_throws_exception_if_next_is_not_in_url_in_GET_request(self):
        """
        method: GET
        context must have next link
        """
        with self.assertRaises(KeyError) as context:
            self.response = self.client.get(
                reverse("translation:translate_form_locale", args=["admin", "testmodel", 1, 'ar']))
            self.assertTrue("key next must be included in request" in str(context.exception))

    def test_it_uses_the_next_locale_if_locale_is_not_provided_in_url(self):
        url = reverse("translation:translate_form",
                      args=["tests", "TranslatableTestModel", self.model.pk]) + "?next=/test/"
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_context_has_target_locale_in_GET_request(self):
        """
        method: GET
        context must have list of languages
        """
        self.response = self.client.get(self.url)
        self.assertIn("locale", self.response.context)

    def test_context_has_app_label_in_GET_request(self):
        """
        method: GET
        context must have app_label
        """
        self.response = self.client.get(self.url)
        self.assertIn("app_label", self.response.context)

    def test_context_has_model_in_GET_request(self):
        """
        method: GET
        context must have model
        """
        self.response = self.client.get(self.url)
        self.assertIn("model", self.response.context)

    def test_context_has_object_in_GET_request(self):
        """
        method: GET
        context must have object
        """
        self.response = self.client.get(self.url)
        self.assertIn("object", self.response.context)

    def test_object_must_be_an_instance_of_model_in_GET_request(self):
        """
        method: GET
        context must have object
        """
        self.response = self.client.get(self.url)
        self.assertEquals(self.response.context['object'], self.model)

    def test_context_has_form_in_GET_request(self):
        """
        method: GET
        context must have form
        """
        self.response = self.client.get(self.url)
        self.assertIn("form", self.response.context)

    def test_form_is_an_instance_of_AdminTranslationForm_in_GET_request(self):
        """
        method: GET
        context must have form
        """
        self.response = self.client.get(self.url)
        self.assertIsInstance(self.response.context["form"], TranslationForm)

    def test_it_throws_exception_if_locale_is_not_in_request_in_POST_request(self):
        """
        method: GET
        context must have next link
        """
        with self.assertRaises(KeyError) as context:
            self.client.post(self.url, {"title": "اهلا وسهلا", "description": "اهلا وسهلا", "save": ""})

    def test_it_throws_exception_if_next_is_not_in_url_in_POST_request(self):
        """
        method: POST
        context must have next link
        """
        with self.assertRaises(KeyError) as context:
            self.client.post(self.url, {
                "title": "اهلا وسهلا",
                "description": "اهلا وسهلا",
                "save": ""})
            self.assertTrue("key next must be included in request" in str(context.exception))

    def test_it_updates_model_value_on_submit_in_POST_request(self):
        self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "description": "اهلا وسهلا",
            "locale": settings.LANGUAGE_ARABIC,
            "next": "/test/",
            "save": ""
        })
        self.model.refresh_from_db()
        self.assertEquals("اهلا وسهلا", self.model.title['ar'])

    def test_it_redirects_to_next_url_if_save_is_not_in_request_in_POST_request(self):
        self.response = self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "description": "اهلا وسهلا",
            "locale": settings.LANGUAGE_ARABIC,
            "next": "/",
        })
        self.assertRedirects(self.response, '/', fetch_redirect_response=False)
        self.assertEquals(self.response.status_code, 302)

    def test_it_redirects_to_same_url_if_save_in_POST_request(self):
        self.response = self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "description": "اهلا وسهلا",
            "next": "/test/",
            "locale": settings.LANGUAGE_ARABIC,
            "save": ""})
        self.assertRedirects(self.response, self.url, fetch_redirect_response=False)
        self.assertEquals(self.response.status_code, 302)

    def test_it_renders_the_same_requests_with_errors_if_data_is_invalid_in_POST_request(self):
        self.response = self.client.post(self.url, {
            "title": "اهلا وسهلا",
            "next": "/test/",
            "locale": settings.LANGUAGE_ARABIC,
            "save": ""})
        self.assertTemplateUsed(self.response, "translation/translate.html")
        self.assertEquals(self.response.status_code, 422)



from django.test import TestCase
from django.urls import reverse, resolve
from translation.views import TranslationAPIView, AdminTranslateView


class TranslationAdminUrlsTestCase(TestCase):
    def test_admin_translate_view_resolve(self):
        url = reverse('translation:translate_form', args=['app_label', 'model', 'pk'])
        self.assertEquals(resolve(url).func.view_class, AdminTranslateView)

    def test_admin_translate_view_resolve_with_local(self):
        url = reverse('translation:translate_form_locale', args=['app_label', 'model', 'pk', 'ar'])
        self.assertEquals(resolve(url).func.view_class, AdminTranslateView)


class TranslationAPIUrlsTestCase(TestCase):
    def test_admin_translate_view_with_locale_resolve(self):
        url = reverse('translation:translate_locale', args=['app_label', 'model', 'pk', 'ar'])
        self.assertEquals(resolve(url).func.view_class, TranslationAPIView)

    def test_admin_translate_view_resolve(self):
        url = reverse('translation:translate', args=['app_label', 'model', 'pk'])
        self.assertEquals(resolve(url).func.view_class, TranslationAPIView)



from django import forms
from django.conf import settings
from django.forms.widgets import TextInput, Textarea
from django.test import TestCase
from django.utils.translation import activate, get_language
from translation.forms import TranslationForm, TranslatableModelForm
from translation.tests.models import TranslatableTestModel as TestModel


class TranslationFormTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

        self.instance = TestModel.objects.create(
            title={"en": "Welcome :)", "ar": "اهلا"},
            description={"en": "Welcome :)", "ar": "اهلا"},
        )

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_updates_the_same_instance_of_the_model(self):
        """
        the form must update the same instance and must not create any new records
        """
        form = TranslationForm(
            model_class=TestModel,
            instance=self.instance,
            locale="ar",
            data={"title": "اهلا", "description": "اهلا"}
        )
        instance = form.save()
        self.assertEquals(instance.pk, self.instance.pk)

    def test_throws_exception_if_model_class_is_not_provided(self):
        """
        model_class must be provided to form
        """
        with self.assertRaises(KeyError) as context:
            TranslationForm(locale="ar", instance=self.instance)
        self.assertTrue("Form missing 1 required key word argument: 'model_class'" in str(context.exception))

    def test_throws_exception_if_instance_is_not_provided(self):
        """
        instance must be provided to form
        """
        with self.assertRaises(KeyError) as context:
            TranslationForm(locale="ar", model_class=TestModel)
        self.assertTrue("Form missing 1 required key word argument: 'instance'" in str(context.exception))

    def test_throws_exception_if_locale_is_not_provided(self):
        """
        locale must be provided to form
        """
        with self.assertRaises(KeyError) as context:
            TranslationForm(model_class=TestModel, instance=self.instance)
        self.assertTrue("Form missing 1 required key word argument: 'locale'" in str(context.exception))

    def test_it_returns_initial_values_with_available_locale_values(self):
        form = TranslationForm(model_class=TestModel, instance=self.instance, locale="ar")
        self.assertEquals(form.initial['title'], "اهلا")
        self.assertEquals(form.initial['description'], "اهلا")

    def test_it_returns_fields_and_widgets_based_on_model_translatable_attributes(self):
        form = TranslationForm(model_class=TestModel, instance=self.instance, locale="ar")
        self.assertIsInstance(form.fields['title'], forms.CharField)
        self.assertIsInstance(form.fields['title'].widget, TextInput)
        self.assertIsInstance(form.fields['description'], forms.CharField)
        self.assertIsInstance(form.fields['description'].widget, Textarea)

    def test_it_updates_fields_without_overriding_other_locales(self):
        self.instance = TestModel.objects.create(title={"en": "Welcome :)"}, description={"en": "Welcome :)"})
        form = TranslationForm(
            model_class=TestModel,
            instance=self.instance,
            locale="ar",
            data={"title": "اهلا", "description": "اهلا"}
        )
        self.assertTrue(form.is_valid())
        form.save()
        self.instance.refresh_from_db()

        self.assertEquals(self.instance.title['en'], "Welcome :)")
        self.assertEquals(self.instance.title['ar'], "اهلا")
        self.assertEquals(self.instance.description['en'], "Welcome :)")
        self.assertEquals(self.instance.description['ar'], "اهلا")

    def test_it_throws_exception_if_input_is_invalid_locales(self):
        """
        throws exception if the locale is not a valid locale
        """
        with self.assertRaises(ValueError) as context:
            TranslationForm(
                model_class=TestModel,
                instance=self.instance,
                locale="not-a-language",
                data={"title": "اهلا", "description": "اهلا"}
            )
            self.assertTrue("input locale must be included in application languages" in str(context.exception))


class TestModelForm(TranslatableModelForm):
    class Meta:
        model = TestModel
        fields = "__all__"


class TranslatableModelFormTestCase(TestCase):
    def setUp(self):
        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
        self.old_FALLBACK_LOCALE = settings.FALLBACK_LOCALE
        self.old_MAIN_LANGUAGE = settings.MAIN_LANGUAGE

        settings.FALLBACK_LOCALE = settings.LANGUAGE_ENGLISH
        settings.LANGUAGE_CODE = settings.LANGUAGE_ENGLISH
        settings.MAIN_LANGUAGE = settings.LANGUAGE_ENGLISH
        activate(settings.LANGUAGE_ENGLISH)

    def tearDown(self):
        settings.FALLBACK_LOCALE = self.old_LANGUAGE_CODE
        settings.LANGUAGE_CODE = self.old_FALLBACK_LOCALE
        settings.MAIN_LANGUAGE = self.old_MAIN_LANGUAGE
        activate(settings.LANGUAGE_CODE)

    def test_it_returns_fields_and_widgets_based_on_model_translatable_attributes(self):
        form = TestModelForm(data={"title": "Welcome :)", "description": "Welcome :)"})
        self.assertIsInstance(form.fields['title'], forms.CharField)
        self.assertIsInstance(form.fields['title'].widget, TextInput)
        self.assertIsInstance(form.fields['description'], forms.CharField)
        self.assertIsInstance(form.fields['description'].widget, Textarea)

    def test_it_returns_field_initial_value_in_original_locale_only(self):
        instance = TestModel.objects.create(
            title={"en": "Welcome from title:)", "ar": "اهلا فى العنوان"},
            description={"en": "Welcome from description:)", "ar": "اهلا فى الوصف"})
        form = TestModelForm(instance=instance)
        self.assertEquals(form.instance.title, "Welcome from title:)")

    def test_it_saves_translatable_value_with_settings_language_code_in_creation_of_new_record(self):
        """
        in creation of new record the original field value is None
        """
        form = TestModelForm(data={"title": "Welcome from title:)", "description": "Welcome from description:)"})
        self.assertTrue(form.is_valid())
        form.save()

        instance = TestModel.objects.get(pk=1)
        self.assertEquals(instance.title['en'], "Welcome from title:)")
        self.assertEquals(instance.description['en'], "Welcome from description:)")

    def test_it_updates_translatable_values_of_settings_default_locale(self):
        """
        in instance update it saves the new values
        """
        TestModel.objects.create(title={"en": "Welcome from title:)"}, description={"en": "Welcome from description:)"})

        form = TestModelForm(data={"title": "Welcome!!", "description": "Welcome Dude!!"},
                             instance=TestModel.objects.get(pk=1))
        self.assertTrue(form.is_valid())
        form.save()
        instance = TestModel.objects.get(pk=1)

        self.assertEquals(instance.title['en'], "Welcome!!")
        self.assertEquals(instance.description['en'], "Welcome Dude!!")

    def test_it_maintains_other_locales_on_instance_update(self):
        """
        in instance update it saves the new values  without changing other locales values
        """
        instance = TestModel.objects.create(
            title={"en": "Welcome from title:)", "ar": "اهلا فى العنوان"},
            description={"en": "Welcome from description:)", "ar": "اهلا فى الوصف"})

        form = TestModelForm(data={"title": "Welcome!!", "description": "Welcome Dude!!"}, instance=instance)

        self.assertTrue(form.is_valid())
        form.save()

        self.assertEquals(instance.title['ar'], "اهلا فى العنوان")
        self.assertEquals(instance.description['ar'], "اهلا فى الوصف")
        self.assertEquals(instance.title['en'], "Welcome!!")
        self.assertEquals(instance.description['en'], "Welcome Dude!!")



from django.shortcuts import render, get_object_or_404, redirect
from django.urls import reverse
from django.views import View
from django.utils.translation import gettext_lazy as _
from django.apps import apps
from django.conf import settings
from django.utils.translation import gettext as _
from rest_framework import status
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response
from rest_framework.views import APIView

from translation.forms import TranslationForm
from translation.serializers import TranslatableModelSerializer


class AdminTranslateView(View):
    def get(self, request, *args, **kwargs):
        if request.GET.get('next') is None:
            raise KeyError(str(_("key next must be included in request")))

        locale = kwargs.get('locale', next(x for x in settings.LANGUAGES_KEYS if x != settings.MAIN_LANGUAGE))
        model = apps.get_model(kwargs['app_label'], kwargs['model'])
        instance = get_object_or_404(model, pk=kwargs['pk'])

        languages = dict(settings.LANGUAGES)
        languages.pop(settings.LANGUAGE_CODE)

        return render(request, "translation/translate.html", context={
            "languages": languages,
            "next": request.GET.get('next'),
            "locale": locale,
            "app_label": kwargs.get("app_label"),
            "model": kwargs.get("model"),
            "object": instance,
            "form": TranslationForm(model_class=model, instance=instance, locale=locale)
        })

    def post(self, request, *args, **kwargs):
        if request.POST.get('next') is None:
            raise KeyError(str(_("key next must be included in request")))

        model = apps.get_model(kwargs['app_label'], kwargs['model'])
        instance = get_object_or_404(model, pk=kwargs['pk'])

        locale = kwargs.get("locale", request.POST.get('locale'))
        next_link = request.POST.get('next')

        form = TranslationForm(model_class=model, instance=instance, locale=locale, data=request.POST)

        if form.is_valid():
            form.save()
            if "save" in request.POST:  # save and go to the next language
                return redirect(reverse('translation:translate_form', kwargs={**kwargs}) + "?next={}".format(next_link))
            else:
                return redirect(to=next_link)

        languages = dict(settings.LANGUAGES)
        languages.pop(settings.LANGUAGE_CODE)

        for field, __ in instance.translatable.items():
            if isinstance(getattr(form.instance, field), str):
                instance.set_translation(field, locale, getattr(form.instance, field), soft=True)

        return render(request, "translation/translate.html", context={
            "languages": languages,
            "next": request.GET.get('next'),
            "locale": kwargs.get("locale"),
            "app_label": kwargs.get("app_label"),
            "model": kwargs.get("model"),
            "object": instance,
            "form": form
        }, content_type=None, status=422)


class TranslationAPIView(APIView):
    def post(self, request, *args, **kwargs):
        model = apps.get_model(kwargs['app_label'], kwargs['model'])
        instance = get_object_or_404(model, pk=kwargs['pk'])
        locale = kwargs.get("locale", request.POST.get('locale'))

        TranslatableModelSerializer.Meta.model = model
        serializer = TranslatableModelSerializer(data=request.data, instance=instance, context={'locale': locale})

        if serializer.is_valid():
            serializer.save()
            return Response(data=serializer.data, status=status.HTTP_200_OK)
        return Response(data=serializer.errors, status=status.HTTP_422_UNPROCESSABLE_ENTITY)



from django import forms
from django.db import models
from django.forms.widgets import Textarea
from django.utils.timezone import now
from translation.models import TranslatableModel


# Translatable Model
class TranslatableTestModel(TranslatableModel):
    translatable = {
        "title": {"field": forms.CharField},
        "description": {"field": forms.CharField, "widget": Textarea},
    }

    id = models.BigAutoField(primary_key=True)
    title = models.JSONField(blank=True, null=True)
    description = models.JSONField(blank=True, null=True)
    created_at = models.DateTimeField(auto_created=True, default=now, blank=True)
    updated_at = models.DateTimeField(auto_now=True, null=True, blank=True)



from django import forms
from django.conf import settings
from django.forms.widgets import TextInput
from django.utils.translation import gettext as _





class TranslationForm(forms.ModelForm):
    fields = {}

    class Meta:
        model = None
        instance = None

    def __init__(self, *args, **kwargs):
        if "locale" not in kwargs:
            raise KeyError(_("Form missing 1 required key word argument: 'locale'"))

        if kwargs.get('locale') not in dict(settings.LANGUAGES).keys():
            raise ValueError(_("input locale must be included in application languages"))

        if "model_class" not in kwargs:
            raise KeyError(_("Form missing 1 required key word argument: 'model_class'"))

        if "instance" not in kwargs:
            raise KeyError(_("Form missing 1 required key word argument: 'instance'"))

        self._meta.instance = kwargs.get('instance')
        self._meta.model = kwargs.pop('model_class')
        self.locale = kwargs.pop('locale')

        initial = {}
        for field_name, __ in self._meta.model.translatable.items():
            field_value = self._meta.instance.get_field_translation(field_name, self.locale)
            initial.update({field_name: field_value})

        kwargs.update(initial=initial)

        super(TranslationForm, self).__init__(*args, **kwargs)

        for name, field in self._meta.model.translatable.items():
            widget = field['widget'] if 'widget' in field else TextInput
            self.fields.update({name: field['field'](widget=widget, required=True, label=_(name))})

    def save(self, commit=True):
        instance = super(TranslationForm, self).save(commit=False)
        instance.refresh_from_db()
        for field in self.cleaned_data:
            instance.set_translation(field, self.locale, self.cleaned_data[field], soft=True)
        if commit:
            instance.save()
        return instance


class TranslatableModelForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        self.original_translatable_values = {}

        if kwargs.get('instance', None):
            instance = kwargs.pop('instance')

            for name, field in instance.translatable.items():
                self.original_translatable_values.update({name: getattr(instance, name)})
                field_value = getattr(instance, name)
                field_initial_value = field_value[
                    settings.MAIN_LANGUAGE] if settings.MAIN_LANGUAGE in field_value else ''
                setattr(instance, name, field_initial_value)
            kwargs.update(instance=instance)

        super(TranslatableModelForm, self).__init__(*args, **kwargs)

        for name, field in self._meta.model.translatable.items():
            widget = field['widget'] if 'widget' in field else forms.TextInput
            self.fields.update({name: field['field'](widget=widget, required=True, label=_(name))})

    def save(self, commit=True):
        instance = super(TranslatableModelForm, self).save(commit=False)

        for field, __ in self._meta.model.translatable.items():
            if field in self.original_translatable_values:
                setattr(instance, field, self.original_translatable_values[field])
            else:
                setattr(instance, field, {})

            if field in self.cleaned_data:
                instance.set_translation(field, settings.LANGUAGE_CODE, self.cleaned_data[field], soft=True)

        if commit:
            instance.save()
        return instance




from django.conf import settings
from django.test import TestCase
from django.urls import reverse
from rest_framework.test import APIClient
from rest_framework import status
from translation.tests.models import TranslatableTestModel as TestModel


class TranslationAPIViewTestCase(TestCase):
    def setUp(self):
        self.test_model = TestModel.objects.create(title={settings.LANGUAGE_CODE: "title"},
                                                   description={"en": "description"})
        self.client = APIClient()
        self.url = reverse('translation:translate', kwargs={"app_label": 'tests',
                                                             'model': 'TranslatableTestModel',
                                                             'pk': '1'})

    def test_it_translate_current_locale_value_when_locale_is_none(self):
        data = {"title": {settings.LANGUAGE_CODE: "other title"}}
        self.client.post(self.url, data=data, format='json')
        self.test_model.refresh_from_db()
        self.assertEquals(self.test_model.title, data['title'])

    def test_it_returns_422_status_code_when_add_invalid_data(self):
        data = {"title": {'sd': "other title"}}
        response = self.client.post(self.url, data=data, format='json')
        self.assertEquals(response.status_code, status.HTTP_422_UNPROCESSABLE_ENTITY)

    def test_it_translate_chosen_locale(self):
        locale = next(locale for locale in settings.LANGUAGES_KEYS if settings.LANGUAGE_CODE != locale)
        data = {"title": {locale: "{} title".format(locale)}, 'locale': locale}
        response = self.client.post(self.url, data=data, format='json')
        self.test_model.refresh_from_db()
        self.assertEquals(self.test_model.title[locale], data["title"][locale])

    def test_it_updates_existed_locales_values(self):
        data = {"title": {"en": "en title", "ar": "ar title"}}
        response = self.client.post(self.url, data=data, format='json')
        self.test_model.refresh_from_db()
        self.assertEquals(self.test_model.title, data['title'])
